/*
 * tmos - pommax2.h
 *
 * Copyright (C) 2009..2012  Krzysztof Mazur <krzysiek@podlesie.net>
 * Copyright (C) 2009  Marek Piwonski <marek@microster.pl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __POMMAX2_H
#define __POMMAX2_H

#include <tmos/kernel.h>
#include <tmos/serial.h>
#include <tmos/spi.h>

#define POMMAX2_CONFIG_MAGIC		0xc000
#define POMMAX2_CONFIG_MAGIC_MASK	0xf000
#define POMMAX2_CONFIG_VERSION		0x0000
#define POMMAX2_CONFIG_VERSION_MASK	0x0f00
#define POMMAX2_CONFIG_REGS_MASK	0x00ff

#define POMMAX2_REG_VERSION	0x00
#define POMMAX2_REG_FREQ	0x01
#define POMMAX2_REG_FLAGS	0x02
#define POMMAX2_REG_OFFSET	0x03
#define POMMAX2_REG_GAIN_0	0x08
#define POMMAX2_REG_GAIN_1	0x09
#define POMMAX2_REG_GAIN_2	0x0a
#define POMMAX2_REG_GAIN_3	0x0b
#define POMMAX2_REG_GAIN_4	0x0c
#define POMMAX2_REG_GAIN_5	0x0d
#define POMMAX2_REG_GAIN_6	0x0e
#define POMMAX2_REG_GAIN_7	0x0f
#define POMMAX2_REG_MAX		0x0f

#define CHANNELS CONFIG_POMMAX2_CHANNELS

/*
 * pommax2_send() is used to send byte.
 */
#if CONFIG_POMMAX2_SPI
#define pommax2_start(c) spi_start(c)
#define pommax2_send(c) spi_send(c)
#define pommax2_stop() spi_stop()
#elif CONFIG_POMMAX2_SERIAL
#define pommax2_start(c) ser_txc(c)
#define pommax2_send(c) ser_txc(c)
#define pommax2_stop() (0)
#else
#define pommax2_start(c) do { (void) (c); } while (0)
#define pommax2_send(c) (0)
#define pommax2_stop() (0)
#endif

void pommax2_send_sample(void) __reentrant;

u16 pommax2_config_read(unsigned char reg) __reentrant;
void pommax2_config_write(u16 v, unsigned char reg) __reentrant;

void pommax2_default_config(void) __reentrant;

#if CONFIG_POMMAX2_NVCONFIG
void pommax2_config_load(void) __reentrant;
char pommax2_config_save(void) __reentrant;
#else
#define pommax2_config_load pommax2_default_config
#define pommax2_config_save() ((char) 0)
#endif

#if CONFIG_POMMAX2_GAIN
int16_t pommax2_gain_apply(int16_t v, unsigned char ch) __reentrant;
void pommax2_gain_configure(void);
#else
#define pommax2_gain_apply(v, ch) ((v) >> 1)
#define pommax2_gain_configure() do { } while (0)
#endif

void pommax2_offset_configure(void);

void pommax2_proto_configure(void) __reentrant;

#endif
