/*
 * tmos - pommax2.c
 * 	version 0.1.0
 * 
 * Copyright (C) 2009..2012  Krzysztof Mazur <krzysiek@podlesie.net>
 * Copyright (C) 2009  Marek Piwonski <marek@microster.pl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <tmos/kernel.h>
#include <tmos/adc.h>
#include "pommax2.h"

#define POMMAX2_SYNC	0xff
#define POMMAX2_STOP	0x00

static u8 __kernel rx[2 * CHANNELS];
static u8 __kernel *rx_ptr;

static unsigned char pommax_send_sample(unsigned char ch)
{
	unsigned char csum = 0;
	int16_t v;
	uint8_t x;

	adc_wait_ch(ch);
	v = adc_get(ch);

	/*
	 * The POMMAX interface allows for 12-bit sample
	 * resolution, but the ADC interface uses 16-bit
	 * format. The four least significant bits are
	 * just skipped.
	 *
	 * TODO: add rounding and error feedback?
	 */
	x = (v >> 10);
	*rx_ptr++ = pommax2_send(x & 0x3f);
	csum += (x & 0x3f);

	x = (v >> 4);
	*rx_ptr++ = pommax2_send(x & 0x3f);
	csum += (x & 0x3f);
	return csum;
}

#define POMMAX3_NOP	0x00
#define POMMAX3_NOP1	0xff
#define POMMAX3_RESET	0x59
#define POMMAX3_WRITE	0x6a
#define POMMAX3_READ	0x33

#define POMMAX3_MAJOR	0x00
#define POMMAX3_MINOR	0x01

#define POMMAX3_FREQ	0x04

#define POMMAX3_ACK	0x24

static u8 ret0;
static u8 ret1;

static void pommax_read(void)
{
	switch (rx[2]) {
	case POMMAX3_MAJOR:
		ret0 = 0;
		break;
	case POMMAX3_MINOR:
		ret0 = 2;
		break;
	}
}

static void pommax_write(void)
{
	switch (rx[2]) {
	case POMMAX3_FREQ:
		adc_set_freq(pack16(rx[4], rx[3]));
		break;
	default:
		ret0 = 0x3f;
	}
}

static u8 old_seq;
static void process_command(void)
{
	u8 x = rx[1];

	if (x == POMMAX3_NOP1)
		return;

	if (x == POMMAX3_RESET) {
		old_seq = 0;
		ret0 = ret1 = 0x3f;
		goto out;
	}

	if (((old_seq ^ x) & 0x80) == 0)
		goto out;
	old_seq = x;

	if (x & 0x80)
		x ^= 0xff;
	if (x == POMMAX3_NOP)
		goto out;

	ret0 = 0;
	ret1 = 0;
	switch (x) {
	case POMMAX3_READ:
		pommax_read();
		break;
	case POMMAX3_WRITE:
		pommax_write();
		break;
	}

out:
	pommax2_send(POMMAX3_ACK);
	pommax2_send(ret0);
	pommax2_send(ret1);
}

void pommax2_send_sample(void) __reentrant
{
	unsigned char i;
	uint8_t csum;

	pommax2_start(POMMAX2_SYNC);
	csum = 0;
	rx_ptr = rx;
	for (i = 0; i < CHANNELS; i++)
		csum += pommax_send_sample(i);
	pommax2_send((~csum) & 0x3f);
	process_command();
	pommax2_send(POMMAX2_STOP);
	pommax2_stop();
}

void pommax2_proto_configure(void) __reentrant
{
}
