/*-------------------------------------------------------------------------
   Register Declarations for the Intel 8051 Processor

   Written By -  Bela Torok / bela.torok@kssg.ch (July 2000)

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   In other words, you are welcome to use, share and improve this program.
   You are forbidden to forbid anyone else to use, share and improve
   what you give them.   Help stamp out software-hoarding!
-------------------------------------------------------------------------*/

#ifndef REG8051_H
#define REG8051_H

#include <asm/sfr.h>

/*  BYTE Register  */
DEFINE_SFR(P0, 0x80)
DEFINE_SFR(SP, 0x81)
DEFINE_SFR(DPL, 0x82)
DEFINE_SFR(DPH, 0x83)
DEFINE_SFR(PCON, 0x87)
DEFINE_SFR(TCON, 0x88)
DEFINE_SFR(TMOD, 0x89)
DEFINE_SFR(TL0, 0x8A)
DEFINE_SFR(TL1, 0x8B)
DEFINE_SFR(TH0, 0x8C)
DEFINE_SFR(TH1, 0x8D)
DEFINE_SFR(P1, 0x90)
DEFINE_SFR(SCON, 0x98)
DEFINE_SFR(SBUF, 0x99)
DEFINE_SFR(P2, 0xA0)
DEFINE_SFR(IE, 0xA8)
DEFINE_SFR(P3, 0xB0)
DEFINE_SFR(IP, 0xB8)
DEFINE_SFR(PSW, 0xD0)
DEFINE_SFR(ACC, 0xE0)
DEFINE_SFR(B, 0xF0)


/*  BIT Register  */
/* P0 */
DEFINE_SBIT(P0_0, 0x80)
DEFINE_SBIT(P0_1, 0x81)
DEFINE_SBIT(P0_2, 0x82)
DEFINE_SBIT(P0_3, 0x83)
DEFINE_SBIT(P0_4, 0x84)
DEFINE_SBIT(P0_5, 0x85)
DEFINE_SBIT(P0_6, 0x86)
DEFINE_SBIT(P0_7, 0x87)

/*  TCON  */
DEFINE_SBIT(IT0, 0x88)
DEFINE_SBIT(IE0, 0x89)
DEFINE_SBIT(IT1, 0x8A)
DEFINE_SBIT(IE1, 0x8B)
DEFINE_SBIT(TR0, 0x8C)
DEFINE_SBIT(TF0, 0x8D)
DEFINE_SBIT(TR1, 0x8E)
DEFINE_SBIT(TF1, 0x8F)

/* P1 */
DEFINE_SBIT(P1_0, 0x90)
DEFINE_SBIT(P1_1, 0x91)
DEFINE_SBIT(P1_2, 0x92)
DEFINE_SBIT(P1_3, 0x93)
DEFINE_SBIT(P1_4, 0x94)
DEFINE_SBIT(P1_5, 0x95)
DEFINE_SBIT(P1_6, 0x96)
DEFINE_SBIT(P1_7, 0x97)

/*  SCON  */
DEFINE_SBIT(RI, 0x98)
DEFINE_SBIT(TI, 0x99)
DEFINE_SBIT(RB8, 0x9A)
DEFINE_SBIT(TB8, 0x9B)
DEFINE_SBIT(REN, 0x9C)
DEFINE_SBIT(SM2, 0x9D)
DEFINE_SBIT(SM1, 0x9E)
DEFINE_SBIT(SM0, 0x9F)

/* P2 */
DEFINE_SBIT(P2_0, 0xA0)
DEFINE_SBIT(P2_1, 0xA1)
DEFINE_SBIT(P2_2, 0xA2)
DEFINE_SBIT(P2_3, 0xA3)
DEFINE_SBIT(P2_4, 0xA4)
DEFINE_SBIT(P2_5, 0xA5)
DEFINE_SBIT(P2_6, 0xA6)
DEFINE_SBIT(P2_7, 0xA7)

/*  IE   */
DEFINE_SBIT(EX0, 0xA8)
DEFINE_SBIT(ET0, 0xA9)
DEFINE_SBIT(EX1, 0xAA)
DEFINE_SBIT(ET1, 0xAB)
DEFINE_SBIT(ES, 0xAC)
DEFINE_SBIT(EA, 0xAF)

/*  P3  */
DEFINE_SBIT(P3_0, 0xB0)
DEFINE_SBIT(P3_1, 0xB1)
DEFINE_SBIT(P3_2, 0xB2)
DEFINE_SBIT(P3_3, 0xB3)
DEFINE_SBIT(P3_4, 0xB4)
DEFINE_SBIT(P3_5, 0xB5)
DEFINE_SBIT(P3_6, 0xB6)
DEFINE_SBIT(P3_7, 0xB7)

DEFINE_SBIT(RXD, 0xB0)
DEFINE_SBIT(TXD, 0xB1)
DEFINE_SBIT(INT0, 0xB2)
DEFINE_SBIT(INT1, 0xB3)
DEFINE_SBIT(T0, 0xB4)
DEFINE_SBIT(T1, 0xB5)
DEFINE_SBIT(WR, 0xB6)
DEFINE_SBIT(RD, 0xB7)

/*  IP   */
DEFINE_SBIT(PX0, 0xB8)
DEFINE_SBIT(PT0, 0xB9)
DEFINE_SBIT(PX1, 0xBA)
DEFINE_SBIT(PT1, 0xBB)
DEFINE_SBIT(PS, 0xBC)

/*  PSW   */
DEFINE_SBIT(P, 0xD0)
DEFINE_SBIT(F1, 0xD1)
DEFINE_SBIT(OV, 0xD2)
DEFINE_SBIT(RS0, 0xD3)
DEFINE_SBIT(RS1, 0xD4)
DEFINE_SBIT(F0, 0xD5)
DEFINE_SBIT(AC, 0xD6)
DEFINE_SBIT(CY, 0xD7)

/* BIT definitions for bits that are not directly accessible */
/* PCON bits */
#define IDL             0x01
#define PD              0x02
#define GF0             0x04
#define GF1             0x08
#define SMOD            0x80

/* TMOD bits */
#define T0_M0           0x01
#define T0_M1           0x02
#define T0_CT           0x04
#define T0_GATE         0x08
#define T1_M0           0x10
#define T1_M1           0x20
#define T1_CT           0x40
#define T1_GATE         0x80

#define T0_MASK         0x0F
#define T1_MASK         0xF0

/* Interrupt numbers: address = (number * 8) + 3 */
#define IE0_VECTOR      0       /* 0x03 external interrupt 0 */
#define TF0_VECTOR      1       /* 0x0b timer 0 */
#define IE1_VECTOR      2       /* 0x13 external interrupt 1 */
#define TF1_VECTOR      3       /* 0x1b timer 1 */
#define SI0_VECTOR      4       /* 0x23 serial port 0 */

#endif
