/*
 * tmos - compiler.h 
 * Copyright (C) 2009  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Fundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef __ASM_I51_COMPILER_H
#define __ASM_I51_COMPILER_H

/*
 * dummy function for compiler barrier, removed by peephole optimizer
 */
#ifdef SDCC
void __barrier(void) __naked;

#define barrier() __barrier()
#else
#define barrier() do { } while (0)
#endif

#ifdef SDCC
#define __using(x) __using x
#define __interrupt(x) __interrupt x
#define __asm__(x...)	__asm x __endasm
#define __INLINE_ASM	1
#else
#define __using(x)	/* nothing */
#define __interrupt(x)	/* nothing */
#define __naked		/* nothing */
#define __reentrant	/* nothing */
#define __bit		char
#define __asm__(x...)	do { } while (0)
#endif

#ifndef SDCC
# if __CHECKER__
#  define __data		__attribute__((address_space(0)))
#  define __idata		__attribute__((address_space(0)))
#  define __xdata		__attribute__((address_space(1)))
#  define __code		__attribute__((address_space(2)))
# else
#  define __data		/* nothing */
#  define __idata		/* nothing */
#  define __xdata		/* nothing */
#  define __code		/* nothing */
# endif
#endif

#if __CHECKER__
#define __icode	/* nothing */
#else
#define __icode __code
#endif

#if __CHECKER__
# define __force	__attribute__((force))
# define __need_lock(x)	__attribute__((context(x,1,1)))
# define __acquires(x)	__attribute__((context(x,0,1)))
# define __releases(x)	__attribute__((context(x,1,0)))
# define __acquire(x)	__context__(x,1)
# define __release(x)	__context__(x,-1)
# define __cond_lock(x,c)	((c) ? ({ __acquire(x); 1; }) : 0)

#ifndef __UCHAR_MAX__
#define __UCHAR_MAX__	255
#endif
#ifndef __UINT_MAX__
#define __UINT_MAX__	65536
#endif
#ifndef __ULONG_MAX__
#define __ULONG_MAX__	4294967295UL
#endif

#ifndef __CHAR_MIN__
#define __CHAR_MIN__	-128
#endif
#ifndef __INT_MIN_
#define __INT_MIN__	-32768
#endif
#ifndef __LONG_MIN__
#define __LONG_MIN__	-2147483648L
#endif
								
#ifndef __CHAR_MAX__
#define __CHAR_MAX__	127
#endif
#ifndef __INT_MAX__
#define __INT_MAX__	32767
#endif
#ifndef __LONG_MAX__
#define __LONG_MAX__	2147483647L
#endif

#ifndef __CHAR_BIT__
#define __CHAR_BIT__	8
#endif

#define INT8_MIN	SCHAR_MIN
#define INT16_MIN	INT_MIN
#define INT32_MIN	LONG_MIN
#define INT8_MAX	SCHAR_MAX
#define INT16_MAX	INT_MAX
#define INT32_MAX	LONG_MAX
								
#else
# define __force	/* nothing */
# define __builtin_warning(x, y...) (1)
# define __need_lock(x)
# define __acquires(x)
# define __releases(x)
# define __acquire(x) (void)0
# define __release(x) (void)0
# define __cond_lock(x,c) (c)
#endif

#if __GNUC__ || __CHECKER__
/*
 * Check at compile time that something is of a particular type.
 * Always evaluates to 1 so you may use it easily in comparisons.
 */
#define typecheck(type,x) \
({	type __dummy; \
	typeof(x) __dummy2; \
	(void)(&__dummy == &__dummy2); \
	1; \
})

/*
 * Check at compile time that 'function' is a certain type, or is a pointer
 * to that type (needs to use typedef for the function type.)
 */
#define typecheck_fn(type,function) \
({	typeof(type) __tmp = function; \
	(void)__tmp; \
})
#else
#define typecheck(type,x)	(1)
#define typecheck_fn(type,function) (1)
#endif

#endif
