/*
 * tmos - serial.h
 *
 * Copyright (C) 2005..2009  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __ASM_SERIAL_H
#define __ASM_SERIAL_H

#include <tmos/config.h>
#include <asm/system.h>
#include <asm/irqs.h>

#if CONFIG_SERIAL_TIMER3
#include <tmos/log2.h>

#define ____ADUC_T3_DIV(speed) \
	ilog2(CONFIG_OSCILLATOR / 16 / (speed))

#define ADUC_T3BAUDEN	0x80

#define __ADUC_T3_DIV(speed) (min(____ADUC_T3_DIV(speed), 15))

#define ADUC_T3_DIV(speed) (__ADUC_T3_DIV(speed) | ADUC_T3BAUDEN)

#define __ADUC_T3_FD(speed) \
	(DIV_ROUND_CLOSEST(2UL * CONFIG_OSCILLATOR \
			   / (1UL << (__ADUC_T3_DIV(speed) - 1)), \
			   (speed)) - 64)

#define ADUC_T3_FD(speed) clamp(__ADUC_T3_FD(speed), 0, 255)

#define CALC_SPEED(speed) \
	((ADUC_T3_DIV(speed) << 8) | (ADUC_T3_FD(speed) & 0xff))

#elif CONFIG_SERIAL_TIMER2
#define CALC_SPEED(speed) \
	(65536UL - (CONFIG_OSCILLATOR + 16UL * (speed)) / (32UL * (speed)))
#else
#define CALC_SPEED(speed) \
	(256 - (CONFIG_OSCILLATOR + 96UL * (speed)) / (192UL * (speed)))
#endif

#define B9600		CALC_SPEED(9600)
#define B19200		CALC_SPEED(19200)
#define B38400		CALC_SPEED(38400)
#define B57600		CALC_SPEED(57600)
#define B115200		CALC_SPEED(115200)
#define B125000		CALC_SPEED(125000)
#define B150000		CALC_SPEED(150000)
#define B230400		CALC_SPEED(230400)

#ifdef CONFIG_SERIAL_I51_STREAM
extern uint8_t __kernel rx_head;
extern uint8_t __kernel rx_tail;
extern uint8_t __kernel tx_head;
extern uint8_t __kernel tx_tail;

/* check if input buffer is nonempty */
#define ser_rx_poll()	 (rx_head != rx_tail)
#else
#define ser_rx_poll()	 (RI)
#endif

#ifdef CONFIG_SERIAL_SEND_RAW
void tx_send_raw(unsigned char x) __reentrant;
void __tx_send_raw(unsigned char x) __using(I51_SERIAL_BANK) __reentrant;
#endif

#endif
