/*
 * ar2-256
 *
 * Auto-Regressive filter, second order with a_0 = 256
 * 
 * Copyright (C) 2010  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * The input data comes from 12-bit ADC.
 * The output is 15-bit.
 */

static int16_t f_y;
static int16_t f_y1;
static int8_t f_e;
#if CONFIG_FILTER_AR2_256_2NDES
static int8_t f_e1;
#endif

extern long ar2_y;
extern int ar2_t;

#define y ar2_y
#define t ar2_t
#define _y _ar2_y
#define _t _ar2_t

static void f_stage1(void) PROCESS_ATTR
{
#if 1
	__bit x;

	t = f_y1 - f_y;
	x = 0;
	if (t < 0) {
		t = -t;
		x = 1;
	}

	B = PARM_A;	/* 3 clocks */
	/* 42 clocks */
#ifdef __INLINE_ASM
	__asm
		mov	a, (_t + 0)		; 2
		mul	ab			; 9
		mov	(_y + 0), a		; 2
		mov	r4, b			; 2
	__endasm;
#endif

	B = PARM_A;	/* 3 clocks */
#ifdef __INLINE_ASM
	__asm
		mov	a, (_t + 1)		; 2
		mul	ab			; 9
		add	a, r4			; 1
		mov	(_y + 1), a		; 2
		clr	a			; 1
		mov	(_y + 3), a		; 2
		addc	a, b			; 2
		mov	(_y + 2), a		; 2
	__endasm;
#endif
	if (!x)
		y = -y;
#else
//	y = 232L * t;
#endif
}

FILTER_CODE
{
	register int t;

	f_stage1();

	/*
	 * Both f_y1 is 15-bit signed value and PARM_B * x are 15-bit
	 * signed values and we firsty substract those values
	 * and later add to y to save one 32-bit operation.
	 */
	t = PARM_B * x - f_y1;
	y += t;

	/* error feedback */
#if CONFIG_FILTER_AR2_256_2NDES
	t = f_e << 1;
	t = -t + f_e1;
	y = y + t;

	f_e1 = f_e;
#else
	y = y - f_e;
#endif
	f_e = (int8_t)(-((uint8_t)y));

	/* division */
	y += 127;
	t = y >> 8;

	/* integrate */
	t = f_y + t;

	if (t > 16383) {
		t = 16383;
	} else if (t < -16384) {
		t = -16384;
	}
	f_y1 = f_y;
	f_y = t;
}
