/*
 * tmos - serial.h
 *
 * Copyright (C) 2005..2012  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __TMOS_SERIAL_H
#define __TMOS_SERIAL_H

#include <tmos/kernel.h>
#include <tmos/jiffies.h>

#include <asm/serial.h>
#include <asm/system.h>

#if CONFIG_BAUDRATE_9600
#define BDEFAULT B9600
#elif CONFIG_BAUDRATE_19200
#define BDEFAULT B19200
#elif CONFIG_BAUDRATE_38400
#define BDEFAULT B38400
#elif CONFIG_BAUDRATE_57600
#define BDEFAULT B57600
#elif CONFIG_BAUDRATE_115200
#define BDEFAULT B115200
#elif CONFIG_BAUDRATE_125000
#define BDEFAULT B125000
#elif CONFIG_BAUDRATE_150000
#define BDEFAULT B150000
#elif CONFIG_BAUDRATE_230400
#define BDEFAULT B230400
#else
#define BDEFAULT B9600
#endif

#ifndef ser_rx_poll
extern char ser_rx_poll(void) __reentrant;
#endif

void ser_rx_wait(void) __preempt;

#ifdef CONFIG_SERIAL_DATA
char ser_rxd(uint8_t __data *buf, unsigned char len) REENTRANT;
char ser_rxd_t(uint8_t __data *buf, unsigned char len, jiffies_t jiffies) REENTRANT;
void ser_txd(const uint8_t __data *buf, char len) REENTRANT;
#endif

#ifdef CONFIG_SERIAL_XDATA
char ser_rxxd(uint8_t __xdata *buf, unsigned char len) REENTRANT;
char ser_rxxd_t(uint8_t __xdata *buf, unsigned char len, jiffies_t jiffies) REENTRANT;
void ser_txxd(const uint8_t __xdata *buf, char len) REENTRANT;
#endif

#ifdef CONFIG_SERIAL_GENERIC
char ser_rx(uint8_t *buf, unsigned char len) REENTRANT;
char ser_rx_t(uint8_t *buf, unsigned char len, jiffies_t jiffies) REENTRANT;
#endif

void ser_txc(char c) __preempt;

#if CONFIG_SERIAL_GENERIC || CONFIG_SERIAL_DEPRECATED
#define SERIAL_WANT_SER_TX		1
void __ser_tx(const uint8_t *buf, char len) REENTRANT;
#define ser_tx(buf, len) __ser_tx((const uint8_t *__force) (buf), (len))
#endif

void ser_flush(void) __preempt;
void ser_set_spd(uint16_t spd) REENTRANT;

#if CONFIG_SERIAL_I51_FLOW_SW
void ser_xoff(void) __reentrant;
#else
#define ser_xoff() do { } while (0)
#endif

void i51_serial_init(void);

#if CONFIG_KERNEL_XDATA
#define ser_rx_kernel(buf, len)		ser_rxxd(buf, len)
#define ser_rx_kernel_t(buf, len, t)	ser_rxxd_t(buf, len, t)
#define ser_tx_kernel(buf, len)		ser_txxd(buf, len)
#elif CONFIG_KERNEL_DATA
#define ser_rx_kernel(buf, len)		ser_rxd(buf, len)
#define ser_rx_kernel_t(buf, len, t)	ser_rxd_t(buf, len, t)
#define ser_tx_kernel(buf, len)		ser_txd(buf, len)
#else
#define ser_rx_kernel(buf, len)		ser_rx(buf, len)
#define ser_rx_kernel_t(buf, len, t)	ser_rx_t(buf, len, t)
#define ser_tx_kernel(buf, len)		ser_tx(buf, len)
#endif

#if CONFIG_USER_XDATA
#define ser_rx_user(buf, len)		ser_rxxd(buf, len)
#define ser_rx_user_t(buf, len, t)	ser_rxxd_t(buf, len, t)
#define ser_tx_user(buf, len)		ser_txxd(buf, len)
#elif CONFIG_USER_DATA
#define ser_rx_user(buf, len)		ser_rxd(buf, len)
#define ser_rx_user_t(buf, len, t)	ser_rxd_t(buf, len, t)
#define ser_tx_user(buf, len)		ser_txd(buf, len)
#else
#define ser_rx_user(buf, len)		ser_rx(buf, len)
#define ser_rx_user_t(buf, len, t)	ser_rx_t(buf, len, t)
#define ser_tx_user(buf, len)		ser_tx(buf, len)
#endif

#ifdef CONFIG_BEER16B
#define dir_tx()	do { P3_4 = 1; } while (0)
#define dir_rx()	do { P3_4 = 0; } while (0)
#endif

#ifdef CONFIG_SERIAL_DEPRECATED
char ser_rxc(void) __preempt;
void ser_txs(const char *str) __preempt;
#endif

#endif
