/*
 * tmos - fork.c
 *
 * Copyright (C) 2011  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <tmos/sched.h>
#include <tmos/kernel.h>
#include <tmos/errno.h>

/**
 * pid_alloc - allocate pid
 * 
 * This functions returns process identifier than can be used
 * to new task or negative error code.
 */
pid_t pid_alloc(void)
{
	unsigned char i;

	preempt_disable();
	for (i = 0; i < CONFIG_TASKS_MAX; i++) {
		if (task_is_dead(&task[i])) {
			set_task_state(&task[i], TASK_UNINTERRUPTIBLE);
			goto out;
		}
	}
	i = -EAGAIN;

out:
	preempt_enable();
	return i;
}
